/**@name LIBRARY: unCMWClient.ctl

@author: Celine VUILLERMOZ (AB-CO-IS)

Creation Date: 07/10/2005 

Modification History: 
  04/03/2012: Herve
  IS-729  unLHCServices  CMWClient: config cfgRdaNames = "pslxmcr:5020" commented out when adding the CMW config in the config file 
 
  08/03/2009: Herve
    - new entry in the config file for RBAC
    
  16/02/2009: Herve
    - new QPS entry in the config file
    
  05/02/2009: Herve
    - new entry in the config file: cfgMonitorOnDNSEx 
    
  29/01/2009: Herve
    - function to handle RDA exception retunred by the WCCOACMWClient driver
    - new entry in the config file: cfgUseRbac 
    
  30/06/2008: Herve
    - bug childPanel: 
      . replace ChildPanel.. by unGraphicalFrame_ChildPanel

	30/04/2007: Herve
		- new function _unCMWClient_setDriverConfig to set the loopTime
		
	28/02/2007: Herve
		- new entry 
		
	13/12/2006: Herve
		- set cfgMaxTime to 10000 and cfgMinTime to 5000
		
	23/11/2006: Herve
		- entry removed in config file
		- new entry in config file
		
	20/11/2006: Herve
		- set cfgRdaTestServ to "UNICOSDEMO"
		
	16/11/2006: Herve
		- set cfgWPLockTime to 1000
		
	04/09/2006: Herve
		- set maxOutput size to 600000 instead of 60000
		- remove: cfgUseAsync = "yes"
		- set cfgMaxTime = "2000"

	27/06/2006: Herve
		- set cfgMaxTime to 15000

	12/06/2006: Herve
		- set cfgInvalidEna to no
		
	30/05/2006: Celine
		- add unCMWClient_deleteDriverConfig function

	19/05/2006: Herve
		- add unCMWClient_setDriverConfig function

version 1.1

External Functions: 
	
Internal Functions: 
	
Purpose: 
This library contains generic functions for CMWClient component.

Usage: Public

PVSS manager usage: CTRL (WCCOACTRL), UI (WCCOANV) and UI extended (WCCOANG)

Constraints:
	. PVSS version: 3.0
	. operating system: Linux, NT, XP and W2000, but tested only under W2000 and XP.
	. distributed system: yes.

	Functions:
		unCMWClient_FilterTreeInit
		unCMWClient_FilterTreeRightClick
		unCMWClient_FilterTreeNodeRightClick
		unCMWClient_FilterTreeLeftClick
		unCMWClient_getFilterName
	*/
	
//Catalogue name
const string g_sCat_CMWClient="unCMWClient";
string classFilter = "Filter";
string classTag = "Tag";
const string cFilterDpe = "CMWClient_Filters.";

// constant for the popupmenu
const int unCMWClient_popupMenu_nodeName=1;
const int unCMWClient_popupMenu_addFilter=2;
const int unCMWClient_popupMenu_editFilter=3;
const int unCMWClient_popupMenu_removeFilter=4;
const int unCMWClient_popupMenu_cutFilter=5;
const int unCMWClient_popupMenu_copyFilter=6;
const int unCMWClient_popupMenu_pasteFilter=7;
const int unCMWClient_popupMenu_addTag=8;
const int unCMWClient_popupMenu_editTag=9;
const int unCMWClient_popupMenu_removeTag=10;
const int unCMWClient_popupMenu_cutTag=11;
const int unCMWClient_popupMenu_copyTag=12;
const int unCMWClient_popupMenu_pasteTag=13;
const string CMW_CLIENT_DEFAULT_LOOPTIME = "200";

mapping g_mFilterList;

//@{

//------------------------------------------------------------------------------------------------------------------------

//unCMWClient_setDriverConfig

/**
Purpose: create (if necessary) all the internal Driver CMW DP and set the default value 

Parameter:
	 	iDriverNumber, int, input, driver number
		exceptionInfo: dyn_string, output, for errors

Usage: Public

PVSS manager usage: Ctrl, UI (WCCOANV) and UI extended (WCCOANG)

Constraints:
	. PVSS version: 3.0
	. operating system: Linux, NT, XP and W2000, but tested only under W2000 and XP.
	. distributed system: yes.
*/
unCMWClient_setDriverConfig(int iDriverNumber, dyn_string &exceptionInfo)
{
	_unCMWClient_setDriverConfig(iDriverNumber, CMW_CLIENT_DEFAULT_LOOPTIME, exceptionInfo);
}

//------------------------------------------------------------------------------------------------------------------------

//_unCMWClient_setDriverConfig

/**
Purpose: create (if necessary) all the internal Driver CMW DP and set the default value 

Parameter:
	 	iDriverNumber, int, input, driver number
		exceptionInfo: dyn_string, output, for errors

Usage: Internal

PVSS manager usage: Ctrl, UI (WCCOANV) and UI extended (WCCOANG)

Constraints:
	. PVSS version: 3.0
	. operating system: Linux, NT, XP and W2000, but tested only under W2000 and XP.
	. distributed system: yes.
*/
_unCMWClient_setDriverConfig(int iDriverNumber, string sLoopTime, dyn_string &exceptionInfo)
{
	dyn_string dsCMWSection = makeDynString(
	"######## Common settings (not always necessary)",
	"maxOutputQueueSize = 600000",
	"######## CMW settings",
	"#CMW network settings",
	"#cfgRdaNames = \"pslxmcr:5020\"",
	"#CMW test server declaration",
	"#cfgRdaTestServ = \"UNICOSDEMO\"",
	"######## Generic settings", 
	"#The min time to repeat CMW connection request (in msec)",
	"cfgMinTime = \"5000\"",		
	"#The max time to repeat CMW connection request (in msec)",
	"cfgMaxTime = \"10000\"",
	"#Time we block PVSS workProc processing when get RDA problems (in msec)",
	"cfgWPLockTime = \"1000\"",		
//	"#Property/tag statistics gathering period",
//	"cfgPropStatTime = \"10000\"",	
//	"#Device/property statistics gathering period",
//	"cfgDevStatTime = \"10000\"",	
	"#Statistics gathering period (in msec)",
	"cfgStatTime = \"10000\"",	
	"#Max receiver cash size (in CMW properties)",
	"cfgMaxRecvSz = \"20000\"",		
	"#Max sender cash size (in CMW properties)",
	"cfgMaxSendSz = \"10000\"",	
//	"#Use CMW async get for PVSS SQ and polling requests",	
//	"cfgUseAsync = \"yes\"",	
	"#DPE invalidation enable",
	"cfgInvalidEna = \"no\"",		
	"#Default Time Calculation Method",
	"cfgDefaultTCM = \"double\"",	
	"#UNICOS Sytem Integrity Check period (in msec)",
	"cfgPeriodManNum = \"10000\"",
	"#CMW monitorOn request on loosing connection",
	"cfgMonitorOffOnDiscon = \"yes\"",
	"#Suspend on backup if >0, keep only last value (in msec)",
	"cfgSuspendOnBackup = \"120000\"",
	"#Number of elements we dump from rdaData::print(...), <1 for all or if omited ",
	"#cfgRdaDebugPrint = \"1\"",
	"#CMW monitorOn onChange default, true if omited",
	"#cfgOnChange = \"true\"",
	"#Time the driver is permitted in the workProc (in msec)",
	"loopTime = "+sLoopTime,
        "#RBAC entry: yes=use RBAC/no=do not use RBAC, (default is no)",
        "#cfgUseRbac = \"yes\"",
        "#Application name for RBAC token, 'default' by default",
        "#cfgAppRbac = \"default\"",
        "#Use test RBAC server, use production issue by default (\"no\")",
        "#cfgTestRbac = \"no\"",
        "#Repeat monitorOn even if device isn't found with the RDA DNS (default is yes)",
        "#cfgMonitorOnDNSEx = \"no\"",
        "######## APM specific settings (not always necessary)", 
        "#QPS data are too far in the future (in msec)", 
        "#cfgQpsInfuture = \"2000\"", 
        "#QPS watchdog timeout (in msec)",
        "#cfgQpsWatchdog = \"10000\"",
        "#QPS data are too far in the past (in msec)",
        "#cfgQpsInpast = \"2000\"",
        "######## end APM specific settings", 
	"");
 _unCMWClient_setDriverConfigSection(iDriverNumber, dsCMWSection, exceptionInfo);
}

//------------------------------------------------------------------------------------------------------------------------

//_unCMWClient_setDriverConfigSection

/**
Purpose: create (if necessary) all the internal Driver CMW DP and set the default value 

Parameter:
	 	iDriverNumber, int, input, driver number
   dsCMWSection, dyn_string, config section data
		exceptionInfo: dyn_string, output, for errors

Usage: Internal

PVSS manager usage: Ctrl, UI (WCCOANV) and UI extended (WCCOANG)

Constraints:
	. PVSS version: 3.0
	. operating system: Linux, NT, XP and W2000, but tested only under W2000 and XP.
	. distributed system: yes.
*/
_unCMWClient_setDriverConfigSection(int iDriverNumber, dyn_string dsCMWSection, dyn_string &exceptionInfo)	
{
	string sDpName, sDriverNumber;
	int iRes;
	if(iDriverNumber<10)	sDriverNumber= "0"+iDriverNumber;
	else	sDriverNumber= iDriverNumber;
  
 	// create (if not existing) the DP CMWClient_Driver_iDriverNumber of type _UnCMWClient_Driver
	// check it exists, if not --> error	
	sDpName = "CMWClient_Driver_" + sDriverNumber;
  if(!dpExists(sDpName))
  {
  	iRes = dpCreate(sDpName, "_Un_CMWClient_Driver");
   	if(!dpExists(sDpName))
				fwException_raise(exceptionInfo, "ERROR", "unCMWClient_setDriverConfig(): cannot create DP"+ sDpName,"");	
		else
			{
				// set the default settintg CMWClient_Driver_iDriverNumber
				iRes = dpSet(sDpName + ".drv_MainArgs", "-m 0x00000007 -t");
				if(iRes < 0) 
					fwException_raise(exceptionInfo, "ERROR", "unCMWClient_setDriverConfig(): cannot set default settings to "+ sDpName,"");	
			}		
	}
	
	// create (if not existing) the DP _Stat_Configs_driver_iDriverNumber of type _Statistics_DriverConfigs
	// check it exists, if not --> error
	sDpName = "_Stat_Configs_driver_" + iDriverNumber;
	if(!dpExists(sDpName))
  {
  	iRes = dpCreate(sDpName, "_Statistics_DriverConfigs");
    if(!dpExists(sDpName))
				fwException_raise(exceptionInfo, "ERROR", "unCMWClient_setDriverConfig(): cannot create DP"+ sDpName,"");	
	}	

	// create (if not existing) the DP _Simulator_iDriverNumber of type _Simulator
	// check it exists, if not --> error
	sDpName = "_Simulator_" + iDriverNumber;
	if(!dpExists(sDpName))
  {
  	iRes = dpCreate(sDpName, "_Simulator");
    if(!dpExists(sDpName))
				fwException_raise(exceptionInfo, "ERROR", "unCMWClient_setDriverConfig(): cannot create DP"+ sDpName,"");	
	}	
	
	// create (if not existing) the DP _Stat_event_0_to_driver_iDriverNumber of type _Statistics_Connections
	// check it exists, if not --> error
	sDpName = "_Stat_event_0_to_driver_" + iDriverNumber;
	if(!dpExists(sDpName))
  {
  	iRes = dpCreate(sDpName, "_Statistics_Connections");
    if(iRes < 0) 
				fwException_raise(exceptionInfo, "ERROR", "unCMWClient_setDriverConfig(): cannot create DP"+ sDpName,"");	
	}	

	// create (if not existing) the DP _DriveriDriverNumber of type _DriverCommon
	// check it exists, if not --> error
	sDpName = "_Driver" + iDriverNumber;
	if(!dpExists(sDpName))
  {
  	iRes = dpCreate(sDpName, "_DriverCommon");
    if(!dpExists(sDpName))
				fwException_raise(exceptionInfo, "ERROR", "unCMWClient_setDriverConfig(): cannot create DP"+ sDpName,"");	
	}	

	// add in the config file the section [CMWClient_iDriverNumber] with the default value
	iRes = fwInstallation_setSection( "CMWClient_"+iDriverNumber, dsCMWSection);
	if(iRes < 0) 
				fwException_raise(exceptionInfo, "ERROR", "unCMWClient_setDriverConfig(): cannot write settings in config file","");	
}

//------------------------------------------------------------------------------------------------------------------------

//unCMWClient_deleteDriverConfig

/**
Purpose: delete (if necessary) all the internal Driver CMW DP 

Parameter:
	 	iDriverNumber, int, input, driver number
		exceptionInfo: dyn_string, output, for errors

Usage: Public

PVSS manager usage: Ctrl, UI (WCCOANV) and UI extended (WCCOANG)

Constraints:
	. PVSS version: 3.0
	. operating system: Linux, NT, XP and W2000, but tested only under W2000 and XP.
	. distributed system: yes.
*/
unCMWClient_deleteDriverConfig(int iDriverNumber, dyn_string &exceptionInfo)
{
	string sDpName, sDriverNumber;
	int iRes;
		
	if(iDriverNumber<10)	sDriverNumber= "0"+iDriverNumber;
	else	sDriverNumber= iDriverNumber;
  
 	// delete (if not existing) the DP CMWClient_Driver_iDriverNumber of type _UnCMWClient_Driver
	// check it exists, if not --> error	
	sDpName = "CMWClient_Driver_" + sDriverNumber;
  if(dpExists(sDpName))
  {
  	iRes = dpDelete(sDpName, "_Un_CMWClient_Driver");
   	if(dpExists(sDpName))
				fwException_raise(exceptionInfo, "ERROR", "unCMWClient_deleteDriverConfig(): cannot delete DP"+ sDpName,"");	
	}
		
	// add in the config file the section [CMWClient_iDriverNumber] with the default value
	iRes = fwInstallation_clearSection( "CMWClient_" + iDriverNumber);
	if(iRes < 0) 
				fwException_raise(exceptionInfo, "ERROR", "unCMWClient_deleteDriverConfig(): cannot delete settings in config file","");	
}

//------------------------------------------------------------------------------------------------------------------------

//unCMWClient_FilterTreeInit

/**
Purpose: initialise the filter active x tree 

Parameter: 	sSelectedKey, string, input, key of the selected filter
			sCompositeLabel, string, input, selected label

Usage: Public

PVSS manager usage: Ctrl, UI (WCCOANV) and UI extended (WCCOANG)

Constraints:
	. PVSS version: 3.0
	. operating system: Linux, NT, XP and W2000, but tested only under W2000 and XP.
	. distributed system: yes.
*/

//unCMWClient_FilterTreeInit(string sSelectedKey, string sCompositeLabel){
unCMWClient_FilterTreeInit(string sGraphTree, string sBar, bool bClipboard)
{
		/*unTreeWidget_classSeparator ~
		unTreeWidget_compositeLabelSeparator |
		unTreeWidget_keySeparator @
		unTreeWidget_labelSeparator ;
		*/
	dyn_string dsFilterList, dsTemp, dsSplit;
	int len, iRes,iCurrentPointer,iMousePointer, id;
	string imageClassParameters, iconWithChildrenClose, iconWithChildrenOpen, iconWithChildrenSelected, 
	iconWithChildrenPath, iconWithoutChildrenNotSelected, iconWithoutChildrenSelected, sFilter,sFilterKey,sTagKey,sSystemName;
	string sSelectedKey;
	string sCompositeLabel;
	shape aShape;
	
	id = unProgressBar_start(sBar);	
	aShape = getShape(sGraphTree);
	iCurrentPointer = unTreeWidget_getPropertyValue(aShape, "getMousePointer", true);
	iMousePointer = unTreeWidget_ccHourglass;
	unTreeWidget_setPropertyValue(aShape, "setMousePointer", iMousePointer, true);
	unTreeWidget_setPropertyValue(aShape, "Enabled", false, true);
	
	iconWithChildrenClose = getPath(PICTURES_REL_PATH,"tag/iconWithChildrenClose.bmp");
	iconWithChildrenOpen = getPath(PICTURES_REL_PATH,"tag/iconWithChildrenOpen.bmp");
	iconWithChildrenSelected = getPath(PICTURES_REL_PATH,"tag/iconWithChildrenSelected.bmp");
	iconWithChildrenPath = getPath(PICTURES_REL_PATH,"tag/iconWithChildrenPath.bmp");
	iconWithoutChildrenNotSelected = getPath(PICTURES_REL_PATH,"tag/iconWithoutChildrenNotSelected.bmp");
	iconWithoutChildrenSelected = getPath(PICTURES_REL_PATH,"tag/iconWithoutChildrenSelected.bmp");
	imageClassParameters = iconWithChildrenClose + ";" + iconWithChildrenOpen + ";" +
									   iconWithChildrenSelected + ";" + iconWithChildrenPath + ";" +
									   iconWithoutChildrenNotSelected + ";" + iconWithoutChildrenSelected;
  unTreeWidget_setPropertyValue(aShape, "addImageClass", classTag+";" + imageClassParameters, true);
  sSystemName=systemName.text;
  
	iRes = dpGet(sSystemName+cFilterDpe, dsFilterList);
	dynSortAsc(dsFilterList);
	for(int i=1;i<=dynlen(dsFilterList); i++)
	{
		dsTemp = strsplit(dsFilterList[i],"/");
		sFilter = dsTemp[1];

		sFilterKey = sFilter+unTreeWidget_labelSeparator +sFilter;
		unTreeWidget_setPropertyValue(aShape, "addTopNode", classFilter+unTreeWidget_classSeparator+unTreeWidget_compositeLabelSeparator +sFilterKey, true);
											
		for(int j=2;j<=dynlen(dsTemp);j++)
			{
				sTagKey = sFilterKey+unTreeWidget_compositeLabelSeparator+dsTemp[j]+unTreeWidget_labelSeparator+dsTemp[j];
				unTreeWidget_setPropertyValue(aShape, "addNode", classTag+unTreeWidget_classSeparator+sTagKey+unTreeWidget_keySeparator+unTreeWidget_compositeLabelSeparator +sFilterKey, true);
//				DebugN(classTag+unTreeWidget_classSeparator+sTagKey+unTreeWidget_keySeparator+unTreeWidget_compositeLabelSeparator +sFilterKey);
			}											
	}	
	unTreeWidget_setPropertyValue(aShape, "set_bNodeNotRemovedOnCollapseExpand", true, true);

	unTreeWidget_setPropertyValue(aShape, "setMousePointer", iCurrentPointer, true);
// enable the unTreeWidget tree.
	unTreeWidget_setPropertyValue(aShape, "Enabled", true, true);
// stop the progress bar.
	unProgressBar_stop(id, sBar);
}

//------------------------------------------------------------------------------------------------------------------------

//unCMWClient_FilterTreeNodeRightClick

/**
Purpose: right click on a node in active x tree

Parameter: 	sSelectedKey, string, input, key of the selected filter
						sMode, string, input, 
						sCompositeLabel, string, input,  selected label	
						sGraphTree, string, input, name of the active x tree
						sBar, string, input,  name of the bar
						bClipboard, boolean, input,  
						iPosX, int, input,  
						iPosY, int,  input, 

Usage: Public

PVSS manager usage: Ctrl, UI (WCCOANV) and UI extended (WCCOANG)

Constraints:
	. PVSS version: 3.0
	. operating system: Linux, NT, XP and W2000, but tested only under W2000 and XP.
	. distributed system: yes.
*/
unCMWClient_FilterTreeNodeRightClick(string sSelectedKey, string sMode, string sCompositeLabel, string sGraphTree, string sBar, bool bClipboard, int iPosX, int iPosY)
{ 
	string sDpName, sDisplayNodeName, sParentDpName, sParentNodeName, sTree, sTagName, sTagKey,	sFilter, sFilterKey, sSystemName;
	dyn_string dsMenu, exInfo, ds, dsTemp, dsText;
	dyn_float df, dfFloat;
	int res, ans, id, pos, iNum, isClipboard, isRoot, iCurrentMousePointer, iMousePointer;
	shape aShape;
	bool isInClipboard;
	mapping mFilterList;
	
// start the progress bar
	id = unProgressBar_start(sBar);	
	aShape = getShape(sGraphTree);
// disable the unTreeWidget tree to have one action at a time.
	unTreeWidget_setPropertyValue(aShape, "Enabled", false, true);
	iCurrentMousePointer = unTreeWidget_getPropertyValue(aShape, "getMousePointer", true);
	iMousePointer = unTreeWidget_ccHourglass;
	unTreeWidget_setPropertyValue(aShape, "setMousePointer", iMousePointer, true);
sSystemName=systemName.text;

	unTreeWidget_getNodeNameAndNodeDp(sCompositeLabel, sDpName, sDisplayNodeName, sParentDpName, sParentNodeName);
				
	if(sParentDpName=="")	//Filter
	{
		dynAppend(dsMenu, "PUSH_BUTTON, "+sDisplayNodeName+", "+unCMWClient_popupMenu_nodeName+", 0");
		dynAppend(dsMenu,"SEPARATOR");
		dynAppend(dsMenu,"PUSH_BUTTON, Add Filter..., "+unCMWClient_popupMenu_addFilter+", 1");		
		dynAppend(dsMenu,"PUSH_BUTTON, Copy Filter..., "+unCMWClient_popupMenu_copyFilter+", 1");		
		dynAppend(dsMenu,"PUSH_BUTTON, Remove Filter..., "+unCMWClient_popupMenu_removeFilter+", 1");
		dynAppend(dsMenu,"SEPARATOR");
		dynAppend(dsMenu,"PUSH_BUTTON, Add Tag..., "+unCMWClient_popupMenu_addTag+", 1");
	}	
	else //Tag
	{
		dynAppend(dsMenu, "PUSH_BUTTON, "+substr(sDisplayNodeName,0,strpos(sDisplayNodeName,","))+", "+unCMWClient_popupMenu_nodeName+", 0");
		dynAppend(dsMenu,"SEPARATOR");
		dynAppend(dsMenu,"PUSH_BUTTON, Edit Tag..., "+unCMWClient_popupMenu_editTag+", 1");
		dynAppend(dsMenu,"PUSH_BUTTON, Remove Tag..., "+unCMWClient_popupMenu_removeTag+", 1");
	}

// show it
	res = popupMenuXY(dsMenu,iPosX, iPosY, ans);

	switch(ans) {
		case unCMWClient_popupMenu_addFilter:
			unGraphicalFrame_ChildPanelOnCentralReturn("vision/unCMWClient/unCMWClientAddFilter.pnl", "Add Filter", makeDynString("$sSystemName:"+sSystemName),df,ds);
			if (ds[1]!="")
			{
				sFilterKey = ds[1]+unTreeWidget_labelSeparator +ds[1];
				unTreeWidget_setPropertyValue(aShape, "addTopNode", classFilter+unTreeWidget_classSeparator+unTreeWidget_compositeLabelSeparator +sFilterKey, true);
				dpGet(sSystemName+cFilterDpe,  dsTemp);
				dynAppend( dsTemp, ds[1]);
				dpSet(sSystemName+cFilterDpe,  dsTemp);
			}
		break;	
		case unCMWClient_popupMenu_removeFilter:
			dsTemp = strsplit(sSelectedKey, ";");
			mFilterList = unCMWClient_getFilterName(aShape);
			if(mappingHasKey(mFilterList,dsTemp[2]))
					sFilter = mFilterList[dsTemp[2]];
				else sFilter=dsTemp[2];	
			dynClear(dsTemp);
			for (int i = 1; i <= mappinglen(mFilterList); i++)
				{
				dsTemp[i] = mappingGetValue(mFilterList,i);
				}
			iNum = dynContains(dsTemp, sFilter);
			dynRemove(dsTemp, iNum);
			unTreeWidget_setPropertyValue(aShape, "deleteNode", sSelectedKey, true);	
			dpSet(sSystemName+cFilterDpe, dsTemp);
			selectedFilter.text	= "";
		break;
		case unCMWClient_popupMenu_addTag:
//		DebugN("addTag", sSystemName);
			unGraphicalFrame_ChildPanelOnCentralReturn("vision/unCMWClient/unCMWClientAddTag.pnl", "Add Tag", makeDynString("$sFilterName:"+sDisplayNodeName,"$sSystemName:"+sSystemName),df,ds);
			if (ds[1]!="")
			{
			sTagName = ds[1];
			sFilterKey = sDisplayNodeName+unTreeWidget_labelSeparator +sDisplayNodeName;
			sTagKey = sFilterKey+unTreeWidget_compositeLabelSeparator+sTagName+unTreeWidget_labelSeparator+sTagName;
			unTreeWidget_setPropertyValue(aShape, "addNode", classTag+unTreeWidget_classSeparator+sTagKey+unTreeWidget_keySeparator+unTreeWidget_compositeLabelSeparator +sFilterKey, true);
			selectedFilter.text	= "";
			}
		break;
		case unCMWClient_popupMenu_editTag:
//		DebugN("editTag", sSystemName);
			dsTemp = strsplit(sSelectedKey, ";");
			mFilterList = unCMWClient_getFilterName(aShape);
			sFilter = mFilterList[dsTemp[1]];
			unGraphicalFrame_ChildPanelOnCentralReturn("vision/unCMWClient/unCMWClientEditTag.pnl", "Edit Tag", makeDynString("$sTagName:"+sDisplayNodeName,"$sFilterName:"+sParentDpName,"$sSystemName:"+sSystemName),df,ds);
			if (ds[1]!="")
			{
				unTreeWidget_setPropertyValue(aShape, "deleteNode", sSelectedKey, true);	
				sTagName = ds[1];
				sFilterKey = sParentNodeName+unTreeWidget_labelSeparator +sParentNodeName;
				sTagKey = sFilterKey+unTreeWidget_compositeLabelSeparator+sTagName+unTreeWidget_labelSeparator+sTagName;
				unTreeWidget_setPropertyValue(aShape, "addNode", classTag+unTreeWidget_classSeparator+sTagKey+unTreeWidget_keySeparator+unTreeWidget_compositeLabelSeparator +sFilterKey, true);
				selectedFilter.text	= "";
			}
			
		break;									
		case unCMWClient_popupMenu_removeTag:
			dsTemp = strsplit(sSelectedKey, ";");
			mFilterList = unCMWClient_getFilterName(aShape);
			sFilter = mFilterList[dsTemp[1]];
			strreplace(sFilter, "/"+sDisplayNodeName, "");
			mFilterList[dsTemp[1]]=sFilter;
			unTreeWidget_setPropertyValue(aShape, "deleteNode", sSelectedKey, true);
			dynClear(dsTemp);
			for (int i = 1; i <= mappinglen(mFilterList); i++)
				{
				dsTemp[i] = mappingGetValue(mFilterList,i);
				}
			dpSet(sSystemName+cFilterDpe, dsTemp);
			selectedFilter.text	= "";
		break;
		
		case unCMWClient_popupMenu_copyFilter:
			unGraphicalFrame_ChildPanelOnCentralModalReturn ("vision/unTreeWidget/unTreeWidgetAddNode.pnl", "Copy Filter", 
																makeDynString("$sTextQuestion:New name for copy of \""+sDisplayNodeName+"\":"), dfFloat, dsText);
			if(dynlen(dfFloat)>0) 
			{
				if(dsText[1] != "") 
				{
					mFilterList = unCMWClient_getFilterName(aShape);
					if(mappingHasKey(mFilterList,dsText[1]))
						unGraphicalFrame_ChildPanelOnCentralModal("vision/MessageWarning","Warning",makeDynString("$1:Creation aborted filterName already exists"));
					else 	unCMWClient_copyTree(sGraphTree, sDisplayNodeName, dsText[1], exInfo);
				}
			}
			
			break;
		default:
		break;
	}

	unTreeWidget_setPropertyValue(aShape, "setMousePointer", iCurrentMousePointer, true);
	unTreeWidget_setPropertyValue(aShape, "Enabled", true, true);
	unProgressBar_stop(id, sBar);

// display the errors here.
	if(dynlen(exInfo) > 0)
		fwExceptionHandling_display(exInfo);
}

//------------------------------------------------------------------------------------------------------------------------

//unCMWClient_FilterTreeLeftClick

/**
Purpose: left click in active x tree

Parameter: 	sSelectedKey, string, input, key of the selected filter
						sMode, string, input, 
						sCompositeLabel, string, input,  selected label	
						sGraphTree, string, input, name of the active x tree
						sBar, string, input,  name of the bar
						bClipboard, boolean, input,  
						iPosX, int, input,  
						iPosY, int,  input, 

Usage: Public

PVSS manager usage: Ctrl, UI (WCCOANV) and UI extended (WCCOANG)

Constraints:
	. PVSS version: 3.0
	. operating system: Linux, NT, XP and W2000, but tested only under W2000 and XP.
	. distributed system: yes.
*/
unCMWClient_FilterTreeLeftClick(string sSelectedKey, string sMode, string sCompositeLabel, string sGraphTree, string sBar, bool bClipboard, int iPosX, int iPosY)
{
	string sDpName, sDisplayNodeName, sParentDpName, sParentNodeName;
	mapping mFilterList;
	shape aShape = getShape(sGraphTree);
	int id, iCurrentMousePointer, iMousePointer;
	
// start the progress bar
	id = unProgressBar_start(sBar);	
	aShape = getShape(sGraphTree);
// disable the unTreeWidget tree to have one action at a time.
	unTreeWidget_setPropertyValue(aShape, "Enabled", false, true);
	iCurrentMousePointer = unTreeWidget_getPropertyValue(aShape, "getMousePointer", true);
	iMousePointer = unTreeWidget_ccHourglass;
	unTreeWidget_setPropertyValue(aShape, "setMousePointer", iMousePointer, true);
	
	unTreeWidget_getNodeNameAndNodeDp(sCompositeLabel, sDpName, sDisplayNodeName, sParentDpName, sParentNodeName);
	mFilterList = unCMWClient_getFilterName(aShape);

	
	if(sParentNodeName!="") 	
		if(mappingHasKey(mFilterList,sParentNodeName)) 
			selectedFilter.text	= mFilterList[sParentNodeName];
		else 
			selectedFilter.text	="";
	else if(mappingHasKey(mFilterList,unTreeWidget_getPropertyValue(aShape, "selectedLabel", true))) 
		selectedFilter.text	= mFilterList[unTreeWidget_getPropertyValue(aShape, "selectedLabel", true)];
	else selectedFilter.text	="";
				
	unTreeWidget_setPropertyValue(aShape, "setMousePointer", iCurrentMousePointer, true);
	unTreeWidget_setPropertyValue(aShape, "Enabled", true, true);
	unProgressBar_stop(id, sBar);			
	
}

//------------------------------------------------------------------------------------------------------------------------

//unCMWClient_getFilterName

/**
Purpose: get the filter name list
Parameter: 	aShape, shape, input, name of the active x tree

Usage: Public

PVSS manager usage: Ctrl, UI (WCCOANV) and UI extended (WCCOANG)

Constraints:
	. PVSS version: 3.0
	. operating system: Linux, NT, XP and W2000, but tested only under W2000 and XP.
	. distributed system: yes.
*/
mapping unCMWClient_getFilterName(shape aShape)
{
	dyn_string dsSplit, dsSplit1, dsSplit2;
	int i;
	string sAllNodeCompositeLabel;
	mapping mFilterList;

	sAllNodeCompositeLabel = unTreeWidget_getPropertyValue(aShape, "getAllNodeCompositeLabel", true);		
	dsSplit = strsplit(sAllNodeCompositeLabel,"@");
//	for(i=1;i<=dynlen(dsSplit);i++)
	for(i=dynlen(dsSplit);i>0;i--)
		{
			dsSplit1 = strsplit(dsSplit[i],"|");
			if(dsSplit1[1]!="")
				{	
					dsSplit2 = strsplit(dsSplit[i],";");
//					DebugN("dsSplit2",dsSplit2);
					if(mappingHasKey(mFilterList,dsSplit2[1]))
						mFilterList[dsSplit2[1]] = mFilterList[dsSplit2[1]]+"/"+ dsSplit2[3];	
//					else mFilterList[dsSplit2[1]] = dsSplit2[1]+"/"+ dsSplit2[3];	
				}	
				else
				{
				dsSplit2 = strsplit(dsSplit1[2],";");
				 mFilterList[dsSplit2[1]] = dsSplit2[1];	
				 }
		}
	return mFilterList;
}

//------------------------------------------------------------------------------------------------------------------------

//unCMWClient_EventRigthClick

/**
Purpose: event when right click on active x tree
Parameter: 	sGraphTree, string, input, name of the graphtree
						sBar, string, input, name of the bar
						bClipBoard, bool, input, true show the clipboard, false don't show the clipboard
						iPosX: int, input: x position of the mouse
						iPosY: int, input: y position of the mouse

Usage: Public

PVSS manager usage: Ctrl, UI (WCCOANV) and UI extended (WCCOANG)

Constraints:
	. PVSS version: 3.0
	. operating system: Linux, NT, XP and W2000, but tested only under W2000 and XP.
	. distributed system: yes.
*/
unCMWClient_EventRigthClick(string sGraphTree, string sBar, bool bClipboard, int iPosX, int iPosY)
{ 
	string sFilterKey, sCopiedNodeName;
	dyn_string dsMenu, exInfo, dsTemp, ds;
	int res, ans;
	dyn_float df;
	shape aShape;
	int id, iCurrentMousePointer, iMousePointer;
	string sCompositeLabel, sNodeDpName, sDisplayNodeName, sParentDpName, sParentNodeName, sTree, sTreeToDelete, sTreeDpToDelete, sSystemName;
	
// start the progress bar
	id = unProgressBar_start(sBar);	
	aShape = getShape(sGraphTree);
// disable the unTreeWidget tree to have one action at a time.
	unTreeWidget_setPropertyValue(aShape, "Enabled", false, true);
	iCurrentMousePointer = unTreeWidget_getPropertyValue(aShape, "getMousePointer", true);
	iMousePointer = unTreeWidget_ccHourglass;
	unTreeWidget_setPropertyValue(aShape, "setMousePointer", iMousePointer, true);
sSystemName=systemName.text;
		
	dynAppend(dsMenu, "PUSH_BUTTON, --- Filter menu ---, "+unCMWClient_popupMenu_nodeName+", 0");
	dynAppend(dsMenu,"SEPARATOR");
	dynAppend(dsMenu,"PUSH_BUTTON, Add Filter, "+unCMWClient_popupMenu_addFilter+", 1");

// show it
	res = popupMenuXY(dsMenu,iPosX, iPosY, ans);

	switch(ans) {
		case unCMWClient_popupMenu_addFilter:
		unGraphicalFrame_ChildPanelOnCentralReturn("vision/unCMWClient/unCMWClientAddFilter.pnl", "Add Filter", makeDynString("$sSystemName:"+sSystemName),df,ds);
			if (ds[1]!="")
			{
				sFilterKey = ds[1]+unTreeWidget_labelSeparator +ds[1];
				unTreeWidget_setPropertyValue(aShape, "addTopNode", classFilter+unTreeWidget_classSeparator+unTreeWidget_compositeLabelSeparator +sFilterKey, true);
				dpGet(sSystemName+cFilterDpe,  dsTemp);
				dynAppend( dsTemp, ds[1]);
				dpSet(sSystemName+cFilterDpe,  dsTemp);
			}
			break;
		default:
			break;
	}

	unTreeWidget_setPropertyValue(aShape, "setMousePointer", iCurrentMousePointer, true);
	unTreeWidget_setPropertyValue(aShape, "Enabled", true, true);
	unProgressBar_stop(id, sBar);

// display the errors here.
	if(dynlen(exInfo) > 0)
		fwExceptionHandling_display(exInfo);
}

//------------------------------------------------------------------------------------------------------------------------

//unCMWClient_copyTree

/**
Purpose: copy the filter to an other name
Parameter: 	sGraphTree, string, input, name of the graphtree
						sTreeToCopy, string, input, name of the copied filter 
						sTreeName, string, input, name of the new filter
						exceptionInfo, dyn_string, output, list of exception

Usage: Public

PVSS manager usage: Ctrl, UI (WCCOANV) and UI extended (WCCOANG)

Constraints:
	. PVSS version: 3.0
	. operating system: Linux, NT, XP and W2000, but tested only under W2000 and XP.
	. distributed system: yes.
*/
unCMWClient_copyTree(string sGraphTree, string sTreeToCopy, string sTreeName, dyn_string &exceptionInfo)
{
	string sFilterKey, sFilter, sTagKey, sNewFilter, sFilterName, sSystemName;
	shape aShape = getShape(sGraphTree);
	mapping	mFilterList = unCMWClient_getFilterName(aShape);
	dyn_string dsTemp;
	int iRes, id, iCurrentMousePointer, iMousePointer;

// start the progress bar
	aShape = getShape(sGraphTree);
// disable the unTreeWidget tree to have one action at a time.
	unTreeWidget_setPropertyValue(aShape, "Enabled", false, true);
	iCurrentMousePointer = unTreeWidget_getPropertyValue(aShape, "getMousePointer", true);
	iMousePointer = unTreeWidget_ccHourglass;
	unTreeWidget_setPropertyValue(aShape, "setMousePointer", iMousePointer, true);

	sFilter= mFilterList[sTreeToCopy];
	dsTemp = strsplit(sFilter,"/");

	sFilterName = sTreeName;
	sSystemName=systemName.text;

	sFilterKey = sFilterName+unTreeWidget_labelSeparator +sFilterName;
	unTreeWidget_setPropertyValue(aShape, "addTopNode", classFilter+unTreeWidget_classSeparator+unTreeWidget_compositeLabelSeparator +sFilterKey, true);
	sNewFilter = sTreeName;											
	for(int j=2;j<=dynlen(dsTemp);j++)
		{
			sNewFilter= sNewFilter+"/"+dsTemp[j];
			sTagKey = sFilterKey+unTreeWidget_compositeLabelSeparator+dsTemp[j]+unTreeWidget_labelSeparator+dsTemp[j];
			unTreeWidget_setPropertyValue(aShape, "addNode", classTag+unTreeWidget_classSeparator+sTagKey+unTreeWidget_keySeparator+unTreeWidget_compositeLabelSeparator +sFilterKey, true);
//				DebugN(classTag+unTreeWidget_classSeparator+sTagKey+unTreeWidget_keySeparator+unTreeWidget_compositeLabelSeparator +sFilterKey);
		}								
	dynClear(dsTemp);	
	dpGet(sSystemName+cFilterDpe,  dsTemp);
	dynAppend( dsTemp, sNewFilter);
	iRes = dpSet(sSystemName+cFilterDpe,  dsTemp);		
	
	if (iRes<0) 
		dynAppend(exceptionInfo, "Error during filter copy");
		
	unTreeWidget_setPropertyValue(aShape, "setMousePointer", iCurrentMousePointer, true);
	unTreeWidget_setPropertyValue(aShape, "Enabled", true, true);

}

//------------------------------------------------------------------------------------------------------------------------

// unCMWClient_getPosition
/**
Purpose: get index for Driver (need for Global dyn)

Parameter: 	sName, string, input, Driver's name
			iIndex, int, output, position in Array of this name
						
Usage: Public

PVSS manager usage: UI (WCCOANV) and UI extended (WCCOANG)

Constraints:
	. PVSS version: 3.0 
	. operating system: W2000, NT, XP and Linux, but tested only under XP and Linux.
	. distributed system: yes.
*/
unCMWClient_getPosition(string sName, int &iIndex)
{
int i, len;

	iIndex=0;
	len=dynlen(g_dsCMWClient_Name);
	for (i=1;i<=len;i++)
		{
		if (g_dsCMWClient_Name[i]==sName)
			{
			iIndex=i;
			i=len+1;
			}
		}		
	
}

//------------------------------------------------------------------------------------------------------------------------

// unCMWClient_getStatus
/**
Purpose: get state of CMWClient system

Parameter: 	iIndex, int, input, Position in Array  
			iState, int, output, State of Manager or Archive  
						
Usage: Public

PVSS manager usage: Ctrl, UI (WCCOANV) and UI extended (WCCOANG)

Constraints:
	. PVSS version: 3.0
	. operating system: Linux, NT, XP and W2000, but tested only under W2000 and XP.
	. distributed system: yes.
*/
unCMWClient_getStatus(int iIndex, int &iState)
{
int iState_1=0, iState_2=0;
	iState=0;
	
	if (dynlen(g_diCMWClient_PVSS)>=iIndex){
			if ((g_diCMWClient_PVSS[iIndex]==0)) 														//Archive or Manager stopped
//				iState_1=1;
					iState=c_Error;
					
			}
			else 		iState=c_Error;
//	if (iState_1==1)	//Error(s)
	
//	DebugN("g_diCMWClient_driver",g_diCMWClient_Driver,  "g_diCMWClient_PVSS", g_diCMWClient_PVSS, "iState", iState);
}


//------------------------------------------------------------------------------------------------------------------------

// unCMWClient_setStatusTable
/**
Purpose: set state of CMWClient system

Parameter: 	bType, bool, input, Manager or Archive
			sName, string, input, Name of Manager or Archive 
			iIndex, int, input, index for Global Variable 
			iState, int, input, Status
			tState_6, int, input, Time of Status change
			dbTimeout, dyn bool, input, True = Warning Timeout or False = Ok
						
Usage: Public

PVSS manager usage: Ctrl, UI (WCCOANV) and UI extended (WCCOANG)

Constraints:
	. PVSS version: 3.0 
	. operating system: W2000, NT, XP and Linux, but tested only under XP and Linux.
	. distributed system: yes.
*/
unCMWClient_setStatusTable(string sName, int iIndex, int iState, time tState_6)
{
int i, length;
string sContent, sTable, sText_PVSS, sColor, t;
dyn_string dsSplitTemp;

	length=DriversList.lineCount();
	sTable="DriversList";
	
for (i=0;i<length;i++)
	{
	getValue(sTable, "cellValueRC", i, "Names", sContent);
	if (sContent==sName)			//find Line
		{
		unCoreLHCServices_PVSS(g_diCMWClient_PVSS[iIndex], sText_PVSS);
		switch(iState)
			{
			case 0:
				sColor="unCMWServer_Ok";
				break;
			
			case c_Warning:
				sColor="unCMWServer_Warning";
				break; 
			
			case c_Error:
				sColor="unCMWServer_Bad";
				break;
			
			default:
				sColor="unCMWServer_Bad";
			}
		
		if (tState_6!="")
			{	
			t=formatTime("%c", tState_6);
	 	
	 		if (_UNIX)
				t+=timeFromGMT(); // ???
      //DebugTN("StateOut", sText_StateOut, sTable, "cellValueRC", i, "MessageOut", sText_Message, sTable, "cellValueRC", i, "Time", t);
 			setMultiValue (sTable, "cellBackColRC", i, "X", sColor, sTable, "cellValueRC", i, "PVSS", sText_PVSS);
 			}
 		else	
			setMultiValue (sTable, "cellBackColRC", i, "X", sColor, sTable, "cellValueRC", i, "PVSS", sText_PVSS);
		
		return;
		}
	}
}

//------------------------------------------------------------------------------------------------------------------------

// unCMWClient_getException
/**
Purpose: get the exception of a RDA set 

Parameter:
  @param exInfo: dyn_string, output: the exception is returned here, ""=OK
  @param iTimeout: int, input: max timeout to wait in sec, default 1sec
					
Usage: Public

PVSS manager usage: Ctrl, UI (WCCOANV) and UI extended (WCCOANG)

Constraints:
  . PVSS version: 3.6 
  . operating system: W2000, NT, XP and Linux, but tested only under XP and Linux.
  . distributed system: yes.
  . global variable in the panel: string g_sDriverDp, g_sDevProp, g_sSetException
*/
unCMWClient_getException(dyn_string &exInfo, int iTimeout=1)
{
  int i, len=iTimeout*2, iPos;
  bool bDone;
  int lenProp = strlen(g_sDevProp)+3;
  string sSetException;
  
  if(dynlen(exInfo) <= 0) {
    sSetException = g_sDevProp+" not executed";
    for(i=0;(i<=len) & !bDone;i++)
    {
      iPos = strpos(g_sSetException, g_sDevProp);
      if(iPos >= 0) {
        bDone = true;
        sSetException = substr(g_sSetException, iPos+lenProp, strlen(g_sSetException) - iPos -lenProp);
        unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "unCMWClient_getException", "unCMWClient_getException", g_sSetException, g_sDevProp, sSetException);
      }
      else
        delay(0, 500);
    }
    dpDisconnect("unCMWClient_ExceptionHandlingCB", g_sDriverDp);
    if(sSetException != "ok")
      fwException_raise(exInfo, "ERROR", sSetException, "");
  }
}

//------------------------------------------------------------------------------------------------------------------------

// unCMWClient_ExceptionHandlingCB
/**
Purpose: Callback function on the cmw_RdaSetMess DPE

Parameter:
  @param sDrvDpe: string, input: the DPE
  @param scmw_RdaSetMess: string, input: the exception is returned here, OK=rdaSend: dev$prop - ok
            ERROR=rdaSend: dev$prop - RDA exception
					
Usage: Public

PVSS manager usage: Ctrl, UI (WCCOANV) and UI extended (WCCOANG)

Constraints:
  . PVSS version: 3.6 
  . operating system: W2000, NT, XP and Linux, but tested only under XP and Linux.
  . distributed system: yes.
  . global variable in the panel: string g_sDriverDp, g_sDevProp, g_sSetException
*/
unCMWClient_ExceptionHandlingCB(string sDrvDpe, string scmw_RdaSetMess)
{
  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "unCMWClient_ExceptionHandlingCB", "unCMWClient_ExceptionHandlingCB", sDrvDpe, g_sDevProp, scmw_RdaSetMess);
  g_sSetException = scmw_RdaSetMess;
}

//------------------------------------------------------------------------------------------------------------------------

// unCMWClient_setUpExceptionHandling
/**
Purpose: set up an exception handler to catch the exception of a RDA set 

Parameter:
  @param sDpe: string, input: the DPE
  @param exInfo: dyn_string, output: the exception is returned here, ""=OK
					
Usage: Public

PVSS manager usage: Ctrl, UI (WCCOANV) and UI extended (WCCOANG)

Constraints:
  . PVSS version: 3.6 
  . operating system: W2000, NT, XP and Linux, but tested only under XP and Linux.
  . distributed system: yes.
  . global variable in the panel: string g_sDriverDp, g_sDevProp, g_sSetException
*/
unCMWClient_setUpExceptionHandling(string sDpe, dyn_string &exInfo)
{
  int i;
  bool bDone;
  bool bConfigExists, bIsActive;
  dyn_anytype daAddressConfig;
  dyn_string exceptionInfo, dsSplit;
  string sCMWSubscription;
  string sSetResult, sDriverNumber;
  string sSystemName = unGenericDpFunctions_getSystemName(sDpe);
  int iDriverNumber;
      
  g_sDriverDp = g_sDevProp = g_sSetException = "";
  fwPeriphAddress_get(sDpe, bConfigExists, daAddressConfig, bIsActive, exceptionInfo);
  if(bConfigExists) {
    iDriverNumber = daAddressConfig[fwPeriphAddress_DRIVER_NUMBER];
    sCMWSubscription = daAddressConfig[fwPeriphAddress_REFERENCE];
    dsSplit = strsplit(sCMWSubscription, "$");
    if(iDriverNumber<10)	
      sDriverNumber= "0"+iDriverNumber;
    else
      sDriverNumber= iDriverNumber;
    g_sDriverDp = sSystemName+"CMWClient_Driver_" + sDriverNumber+".cmw_RdaSetMess";
  
    if(dynlen(dsSplit) >=3) {
      g_sDevProp = dsSplit[1]+"$"+dsSplit[2];
      dpConnect("unCMWClient_ExceptionHandlingCB", 0, g_sDriverDp);
    }
    else
      fwException_raise(exInfo, "ERROR", "Wrong device$property: "+sCMWSubscription, "");
  }
  else
    fwException_raise(exInfo, "ERROR", "No Address config: "+sDpe, "");
  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "unCMWClient_setUpExceptionHandling", "unCMWClient_setUpExceptionHandling", g_sDevProp, g_sSetException, sDpe);
}

//------------------------------------------------------------------------------------------------------------------------

//@}


